const express = require('express');

const ClientsService = require('../services/clients');
const ClientsDBApi = require('../db/api/clients');
const wrapAsync = require('../helpers').wrapAsync;

const router = express.Router();

const { parse } = require('json2csv');

const { checkCrudPermissions } = require('../middlewares/check-permissions');

router.use(checkCrudPermissions('clients'));

/**
 *  @swagger
 *  components:
 *    schemas:
 *      Clients:
 *        type: object
 *        properties:

 *          thyme_id:
 *            type: string
 *            default: thyme_id
 *          name:
 *            type: string
 *            default: name
 *          office:
 *            type: string
 *            default: office
 *          address:
 *            type: string
 *            default: address
 *          city:
 *            type: string
 *            default: city
 *          post_code:
 *            type: string
 *            default: post_code
 *          country_txt:
 *            type: string
 *            default: country_txt
 *          phone_number:
 *            type: string
 *            default: phone_number
 *          website:
 *            type: string
 *            default: website
 *          currency:
 *            type: string
 *            default: currency
 *          curious_customer_id:
 *            type: string
 *            default: curious_customer_id
 *          agency:
 *            type: string
 *            default: agency
 *          client_id_link:
 *            type: string
 *            default: client_id_link
 *          public_url:
 *            type: string
 *            default: public_url
 *          xero_id:
 *            type: string
 *            default: xero_id

 *          early_payment_discount:
 *            type: integer
 *            format: int64
 *          production_fee:
 *            type: integer
 *            format: int64
 *          insurance_fee:
 *            type: integer
 *            format: int64

 */

/**
 *  @swagger
 * tags:
 *   name: Clients
 *   description: The Clients managing API
 */

/**
 *  @swagger
 *  /api/clients:
 *    post:
 *      security:
 *        - bearerAuth: []
 *      tags: [Clients]
 *      summary: Add new item
 *      description: Add new item
 *      requestBody:
 *        required: true
 *        content:
 *          application/json:
 *            schema:
 *              properties:
 *                data:
 *                  description: Data of the updated item
 *                  type: object
 *                  $ref: "#/components/schemas/Clients"
 *      responses:
 *        200:
 *          description: The item was successfully added
 *          content:
 *            application/json:
 *              schema:
 *                $ref: "#/components/schemas/Clients"
 *        401:
 *          $ref: "#/components/responses/UnauthorizedError"
 *        405:
 *          description: Invalid input data
 *        500:
 *          description: Some server error
 */
router.post(
  '/',
  wrapAsync(async (req, res) => {
    const referer =
      req.headers.referer ||
      `${req.protocol}://${req.hostname}${req.originalUrl}`;
    const link = new URL(referer);
    await ClientsService.create(
      req.body.data,
      req.currentUser,
      true,
      link.host,
    );
    const payload = true;
    res.status(200).send(payload);
  }),
);

/**
 * @swagger
 * /api/budgets/bulk-import:
 *  post:
 *    security:
 *      - bearerAuth: []
 *    tags: [Clients]
 *    summary: Bulk import items
 *    description: Bulk import items
 *    requestBody:
 *      required: true
 *      content:
 *        application/json:
 *         schema:
 *          properties:
 *            data:
 *              description: Data of the updated items
 *              type: array
 *              items:
 *                $ref: "#/components/schemas/Clients"
 *    responses:
 *      200:
 *        description: The items were successfully imported
 *    content:
 *      application/json:
 *        schema:
 *          $ref: "#/components/schemas/Clients"
 *      401:
 *        $ref: "#/components/responses/UnauthorizedError"
 *      405:
 *        description: Invalid input data
 *      500:
 *        description: Some server error
 *
 */
router.post(
  '/bulk-import',
  wrapAsync(async (req, res) => {
    const referer =
      req.headers.referer ||
      `${req.protocol}://${req.hostname}${req.originalUrl}`;
    const link = new URL(referer);
    await ClientsService.bulkImport(req, res, true, link.host);
    const payload = true;
    res.status(200).send(payload);
  }),
);

/**
 *  @swagger
 *  /api/clients/{id}:
 *    put:
 *      security:
 *        - bearerAuth: []
 *      tags: [Clients]
 *      summary: Update the data of the selected item
 *      description: Update the data of the selected item
 *      parameters:
 *        - in: path
 *          name: id
 *          description: Item ID to update
 *          required: true
 *          schema:
 *            type: string
 *      requestBody:
 *        description: Set new item data
 *        required: true
 *        content:
 *          application/json:
 *            schema:
 *              properties:
 *                id:
 *                  description: ID of the updated item
 *                  type: string
 *                data:
 *                  description: Data of the updated item
 *                  type: object
 *                  $ref: "#/components/schemas/Clients"
 *              required:
 *                - id
 *      responses:
 *        200:
 *          description: The item data was successfully updated
 *          content:
 *            application/json:
 *              schema:
 *                $ref: "#/components/schemas/Clients"
 *        400:
 *          description: Invalid ID supplied
 *        401:
 *          $ref: "#/components/responses/UnauthorizedError"
 *        404:
 *          description: Item not found
 *        500:
 *          description: Some server error
 */
router.put(
  '/:id',
  wrapAsync(async (req, res) => {
    await ClientsService.update(req.body.data, req.body.id, req.currentUser);
    const payload = true;
    res.status(200).send(payload);
  }),
);

/**
 * @swagger
 *  /api/clients/{id}:
 *    delete:
 *      security:
 *        - bearerAuth: []
 *      tags: [Clients]
 *      summary: Delete the selected item
 *      description: Delete the selected item
 *      parameters:
 *        - in: path
 *          name: id
 *          description: Item ID to delete
 *          required: true
 *          schema:
 *            type: string
 *      responses:
 *        200:
 *          description: The item was successfully deleted
 *          content:
 *            application/json:
 *              schema:
 *                $ref: "#/components/schemas/Clients"
 *        400:
 *          description: Invalid ID supplied
 *        401:
 *          $ref: "#/components/responses/UnauthorizedError"
 *        404:
 *          description: Item not found
 *        500:
 *          description: Some server error
 */
router.delete(
  '/:id',
  wrapAsync(async (req, res) => {
    await ClientsService.remove(req.params.id, req.currentUser);
    const payload = true;
    res.status(200).send(payload);
  }),
);

/**
 *  @swagger
 *  /api/clients/deleteByIds:
 *    post:
 *      security:
 *        - bearerAuth: []
 *      tags: [Clients]
 *      summary: Delete the selected item list
 *      description: Delete the selected item list
 *      requestBody:
 *        required: true
 *        content:
 *          application/json:
 *            schema:
 *              properties:
 *                ids:
 *                  description: IDs of the updated items
 *                  type: array
 *      responses:
 *        200:
 *          description: The items was successfully deleted
 *          content:
 *            application/json:
 *              schema:
 *                $ref: "#/components/schemas/Clients"
 *        401:
 *          $ref: "#/components/responses/UnauthorizedError"
 *        404:
 *          description: Items not found
 *        500:
 *          description: Some server error
 */
router.post(
  '/deleteByIds',
  wrapAsync(async (req, res) => {
    await ClientsService.deleteByIds(req.body.data, req.currentUser);
    const payload = true;
    res.status(200).send(payload);
  }),
);

/**
 *  @swagger
 *  /api/clients:
 *    get:
 *      security:
 *        - bearerAuth: []
 *      tags: [Clients]
 *      summary: Get all clients
 *      description: Get all clients
 *      responses:
 *        200:
 *          description: Clients list successfully received
 *          content:
 *            application/json:
 *              schema:
 *                type: array
 *                items:
 *                  $ref: "#/components/schemas/Clients"
 *        401:
 *          $ref: "#/components/responses/UnauthorizedError"
 *        404:
 *          description: Data not found
 *        500:
 *          description: Some server error
 */
router.get(
  '/',
  wrapAsync(async (req, res) => {
    const filetype = req.query.filetype;

    const currentUser = req.currentUser;
    const payload = await ClientsDBApi.findAll(req.query, { currentUser });
    if (filetype && filetype === 'csv') {
      const fields = [
        'id',
        'thyme_id',
        'name',
        'office',
        'address',
        'city',
        'post_code',
        'country_txt',
        'phone_number',
        'website',
        'currency',
        'curious_customer_id',
        'agency',
        'client_id_link',
        'public_url',
        'xero_id',

        'early_payment_discount',
        'production_fee',
        'insurance_fee',
      ];
      const opts = { fields };
      try {
        const csv = parse(payload.rows, opts);
        res.status(200).attachment(csv);
        res.send(csv);
      } catch (err) {
        console.error(err);
      }
    } else {
      res.status(200).send(payload);
    }
  }),
);

/**
 *  @swagger
 *  /api/clients/count:
 *    get:
 *      security:
 *        - bearerAuth: []
 *      tags: [Clients]
 *      summary: Count all clients
 *      description: Count all clients
 *      responses:
 *        200:
 *          description: Clients count successfully received
 *          content:
 *            application/json:
 *              schema:
 *                type: array
 *                items:
 *                  $ref: "#/components/schemas/Clients"
 *        401:
 *          $ref: "#/components/responses/UnauthorizedError"
 *        404:
 *          description: Data not found
 *        500:
 *          description: Some server error
 */
router.get(
  '/count',
  wrapAsync(async (req, res) => {
    const currentUser = req.currentUser;
    const payload = await ClientsDBApi.findAll(req.query, null, {
      countOnly: true,
      currentUser,
    });

    res.status(200).send(payload);
  }),
);

/**
 *  @swagger
 *  /api/clients/autocomplete:
 *    get:
 *      security:
 *        - bearerAuth: []
 *      tags: [Clients]
 *      summary: Find all clients that match search criteria
 *      description: Find all clients that match search criteria
 *      responses:
 *        200:
 *          description: Clients list successfully received
 *          content:
 *            application/json:
 *              schema:
 *                type: array
 *                items:
 *                  $ref: "#/components/schemas/Clients"
 *        401:
 *          $ref: "#/components/responses/UnauthorizedError"
 *        404:
 *          description: Data not found
 *        500:
 *          description: Some server error
 */
router.get('/autocomplete', async (req, res) => {
  const payload = await ClientsDBApi.findAllAutocomplete(
    req.query.query,
    req.query.limit,
    req.query.offset,
  );

  res.status(200).send(payload);
});

/**
 * @swagger
 *  /api/clients/{id}:
 *    get:
 *      security:
 *        - bearerAuth: []
 *      tags: [Clients]
 *      summary: Get selected item
 *      description: Get selected item
 *      parameters:
 *        - in: path
 *          name: id
 *          description: ID of item to get
 *          required: true
 *          schema:
 *            type: string
 *      responses:
 *        200:
 *          description: Selected item successfully received
 *          content:
 *            application/json:
 *              schema:
 *                $ref: "#/components/schemas/Clients"
 *        400:
 *          description: Invalid ID supplied
 *        401:
 *          $ref: "#/components/responses/UnauthorizedError"
 *        404:
 *          description: Item not found
 *        500:
 *          description: Some server error
 */
router.get(
  '/:id',
  wrapAsync(async (req, res) => {
    const payload = await ClientsDBApi.findBy({ id: req.params.id });

    res.status(200).send(payload);
  }),
);

router.use('/', require('../helpers').commonErrorHandler);

module.exports = router;
